@@ -1,5 +1,4 @@
 import platform
-from pathlib import Path
 from typing import Union, Optional, cast, List, Dict
 
 from easyshare.common import TransferProtocol, TransferDirection, APP_VERSION
@@ -8,13 +7,12 @@ from easyshare.es.errors import ClientErrors
 from easyshare.logging import get_logger
 from easyshare.protocol.requests import Requests, create_request, RequestsParams
 from easyshare.protocol.responses import Response, is_success_response, create_error_response, is_error_response, \
-    ServerErrors, create_success_response
-from easyshare.protocol.types import ServerInfoFull, ServerInfo, FileType
+    ServerErrors, create_success_response, ResponsesParams, is_data_response
+from easyshare.protocol.types import ServerInfoFull, ServerInfo, FileType, SharingInfo
 from easyshare.sockets import SocketTcp, SocketTcpOut
 from easyshare.ssl import get_ssl_context, set_ssl_context
 from easyshare.streams import TcpStream
-from easyshare.tracing import trace_text, get_tracing_level, TRACING_BIN_PAYLOADS
-from easyshare.utils.env import is_windows, is_unix
+from easyshare.tracing import trace_json
 from easyshare.utils.inspection import stacktrace
 from easyshare.utils.json import j, jtob, btoj
 from easyshare.utils.ssl import create_client_ssl_context
@@ -29,12 +27,12 @@ def require_server_connection(api):
     Raises a NOT_CONNECTED if is_connected() is False.
     """
     def require_server_connection_wrapper(conn: 'ConnectionMinimal', *vargs, **kwargs) -> Response:
-        log.d("require_server_connection check before invoking '%s'", api.__name__)
+        log.d(f"require_server_connection check before invoking '{api.__name__}'")
         if not conn.is_connected_to_server():
             log.w("require_server_connection: FAILED")
             log.w(stacktrace(color=ansi.FG_YELLOW))
             return create_error_response(ClientErrors.NOT_CONNECTED)
-        log.d("require_connected_connection: OK - invoking '%s'", api.__name__)
+        log.d(f"require_connected_connection: OK - invoking '{api.__name__}'")
         return api(conn, *vargs, **kwargs)
 
     return require_server_connection_wrapper
@@ -47,12 +45,12 @@ def require_sharing_connection(api):
     Raises a NOT_CONNECTED if is_connected() is False.
     """
     def require_sharing_connection_wrapper(conn: 'ConnectionMinimal', *vargs, **kwargs) -> Response:
-        log.d("require_sharing_connection check before invoking '%s'", api.__name__)
+        log.d(f"require_sharing_connection check before invoking '{api.__name__}'")
         if not conn.is_connected_to_sharing():
             log.w("require_sharing_connection: FAILED")
             log.w(stacktrace(color=ansi.FG_YELLOW))
             return create_error_response(ClientErrors.NOT_CONNECTED)
-        log.d("require_sharing_connection: OK - invoking '%s'", api.__name__)
+        log.d(f"require_sharing_connection: OK - invoking '{api.__name__}'")
         return api(conn, *vargs, **kwargs)
 
     return require_sharing_connection_wrapper
@@ -65,9 +63,9 @@ def handle_connection_response(api):
     """
 
     def handle_connection_response_wrapper(conn: 'ConnectionMinimal', *vargs, **kwargs) -> Response:
-        log.d("Invoking '%s' and handling response", api.__name__)
+        log.d(f"Invoking '{api.__name__}' and handling response")
         resp = api(conn, *vargs, **kwargs)
-        log.d("Handling '%s' response", api.__name__)
+        log.d(f"Handling '{api.__name__}' response")
         if is_error_response(resp, ServerErrors.NOT_CONNECTED):
             log.e("Detected NOT_CONNECTED response, destroying connection")
             conn.destroy_connection()
@@ -81,13 +79,17 @@ def handle_connection_response(api):
 # =============================================
 
 class ConnectionMinimal:
+    """
+    Minimal server connection, containing only the info necessary for actually
+    perform the connection (i.e. ip, port), contrary to 'Connection' which provides
+    ServerInfo too.
+    """
     def __init__(self,
                  server_ip: str, server_port: int, server_ssl: bool,
                  server_alias: str = None, socket: SocketTcp = None):
         # Might throw ConnectionRefusedError
-        log.i("Initializing new ServerConnection %s:%d%s (SSL=%s)",
-              server_ip, server_port, "(" + server_alias + ")" if server_alias else "",
-              server_ssl)
+        log.i(f"Initializing new ServerConnection {server_ip}:{server_port}"
+              f"{'(' + server_alias + ')' if server_alias else ''} (SSL={server_ssl})")
 
         self._server_ip = server_ip
         self._server_port = server_port
@@ -95,7 +97,7 @@ class ConnectionMinimal:
 
         self._connected_to_server: bool = False
         self._connected_to_sharing: bool = False
-        self._sharing_name: Optional[str] = None
+        self._sharing_info: Optional[SharingInfo] = None
         self._rcwd: Optional[str] = None
 
         # SSL setting
@@ -148,18 +150,27 @@ class ConnectionMinimal:
     def is_connected_to_sharing(self) -> bool:
         return True if self.is_connected_to_server() and self._connected_to_sharing else False
 
+    def current_sharing_info(self) -> Optional[SharingInfo]:
+        """ Current remote sharing info """
+        return self._sharing_info if self.is_connected_to_sharing() else None
+
     def current_sharing_name(self) -> Optional[str]:
         """ Current remote sharing name """
-        return self._sharing_name if self.is_connected_to_sharing() else None
+        return self.current_sharing_info().get("name")
 
     def current_rcwd(self) -> Optional[str]:
         """ Current remote working directory (cached) """
         return self._rcwd if self.is_connected_to_sharing() else None
 
-    def destroy_connection(self):
-        log.d("Destroying connection")
-        self.destroy_sharing_connection()
-        self.destroy_server_connection()
+    def destroy_connection(self, clean: bool=True):
+        log.d(f"Destroying connection, clean={clean}")
+
+        if clean:
+            # Clean means to send CLOSE and DISCONNECT as expected
+            self.destroy_sharing_connection()
+            self.destroy_server_connection()
+        # else: just shutdown the socket, although the server won't be happy
+
         self._destroy_stream()
 
 
@@ -190,7 +201,7 @@ class ConnectionMinimal:
                       "invaliding it anyway")
 
         self._connected_to_sharing = False
-        self._sharing_name = None
+        self._sharing_info = None
         self._rcwd = None
         return resp
 
@@ -255,9 +266,9 @@ class ConnectionMinimal:
             RequestsParams.OPEN_SHARING: sharing_name
         }))
 
-        if is_success_response(resp):
+        if is_data_response(resp):
             self._connected_to_sharing = True
-            self._sharing_name = sharing_name
+            self._sharing_info = resp["data"]
             self._rcwd = "/"
         # else?
 
@@ -265,16 +276,11 @@ class ConnectionMinimal:
 
     @handle_connection_response
     @require_server_connection
-    def rexec(self, cmd: str) -> Response:
-        return self.call(create_request(Requests.REXEC, {
-            RequestsParams.REXEC_CMD: cmd
-        }))
-
-    @handle_connection_response
-    @require_server_connection
-    def rshell(self, cmd: str) -> Response:
+    def rshell(self, cmd: str, cols: int, rows: int) -> Response:
         return self.call(create_request(Requests.RSHELL, {
-            RequestsParams.RSHELL_CMD: cmd
+            RequestsParams.RSHELL_CMD: cmd,
+            RequestsParams.RSHELL_COLS: cols,
+            RequestsParams.RSHELL_ROWS: rows
         }))
 
     # === REAL SHARING FUNCS ===
@@ -301,6 +307,13 @@ class ConnectionMinimal:
 
         return resp
 
+    @handle_connection_response
+    @require_sharing_connection
+    def rstat(self, paths: List[str]) -> Response:
+        return self.call(create_request(Requests.RSTAT, {
+            RequestsParams.RSTAT_PATHS: paths
+        }))
+
     @handle_connection_response
     @require_sharing_connection
     def rls(self, sort_by: List[str], reverse: bool = False,
@@ -329,7 +342,8 @@ class ConnectionMinimal:
     @handle_connection_response
     @require_sharing_connection
     def rfind(self, name: str = None, regex: str = None, case_sensitive: bool = True,
-              ftype: FileType = None, details: bool = False, path: str = None) -> Response:
+              ftype: FileType = None, details: bool = False, path: str = None,
+              max_depth: int = None) -> Response:
 
         return self.call(create_request(Requests.RFIND, {
             RequestsParams.RFIND_PATH: path,
@@ -338,6 +352,7 @@ class ConnectionMinimal:
             RequestsParams.RFIND_CASE_SENSITIVE: case_sensitive,
             RequestsParams.RFIND_FTYPE: ftype,
             RequestsParams.RFIND_DETAILS: details,
+            RequestsParams.RFIND_MAX_DEPTH: max_depth,
         }))
 
     @handle_connection_response
@@ -404,11 +419,14 @@ class ConnectionMinimal:
 
     @handle_connection_response
     @require_sharing_connection
-    def put(self, check: bool, sync: bool, preview: bool) -> Response:
+    def put(self, check: bool, preview: bool,
+            dest: Optional[str] = None,
+            is_multiple: Optional[bool] = None) -> Response:
         return self.call(create_request(Requests.PUT, {
             RequestsParams.PUT_CHECK: check,
-            RequestsParams.PUT_SYNC: sync,
             RequestsParams.PUT_PREVIEW: preview,
+            RequestsParams.PUT_DEST: dest,
+            RequestsParams.PUT_IS_MULTIPLE: is_multiple,
         }))
 
 
@@ -419,31 +437,21 @@ class ConnectionMinimal:
         self.write_json(req)
         return self.read_json()
 
-    def write_json(self, req: Dict, trace: bool = True):
-        # Trace OUT
-        trace_bin = get_tracing_level() >= TRACING_BIN_PAYLOADS
+    def write_json(self, req: Dict):
+        trace_json(req,
+                   sender=self._stream.endpoint(), receiver=self._stream.remote_endpoint(),
+                   direction=TransferDirection.OUT, protocol=TransferProtocol.TCP)
 
-        if trace and not trace_bin:
-            trace_text(
-                j(req),
-                sender=self._stream.endpoint(), receiver=self._stream.remote_endpoint(),
-                direction=TransferDirection.OUT, protocol=TransferProtocol.TCP
-            )
+        self.write(jtob(req), trace=False)
 
-        self.write(jtob(req), trace=trace and trace_bin)
+    def read_json(self) -> Dict:
+        resp = btoj(self.read(trace=False))
 
-    def read_json(self, trace: bool = True) -> Dict:
-        trace_bin = get_tracing_level() >= TRACING_BIN_PAYLOADS
-
-        resp = btoj(self.read(trace=trace and trace_bin))
-
-        # Trace IN
-        if trace and not trace_bin:
-            trace_text(
-                j(resp),
-                sender=self._stream.remote_endpoint(), receiver=self._stream.endpoint(),
-                direction=TransferDirection.IN, protocol=TransferProtocol.TCP
-            )
+        trace_json(
+            resp,
+            sender=self._stream.remote_endpoint(), receiver=self._stream.endpoint(),
+            direction=TransferDirection.IN, protocol=TransferProtocol.TCP
+        )
 
         return resp
 
@@ -454,6 +462,12 @@ class ConnectionMinimal:
 
         try:
             self._stream.write(data, trace=trace)
+        except KeyboardInterrupt as kex:
+            # Pass the KeyboardInterrupt above, so that it could be handled
+            # it in a different manner than just shutdown the connection
+            log.w("CTRL+C while writing to socket, "
+                  "it will be probably remain in an inconsistent state")
+            raise kex
         except:
             self._destroy_stream()
             raise ConnectionError("Write failed")
@@ -465,6 +479,12 @@ class ConnectionMinimal:
 
         try:
             return self._stream.read(trace=trace)
+        except KeyboardInterrupt as kex:
+            # Pass the KeyboardInterrupt above, so that it could be handled
+            # it in a different manner than just shutdown the connection
+            log.w("CTRL+C while reading from socket, "
+                  "it will be probably remain in an inconsistent state")
+            raise kex
         except:
             self._destroy_stream()
             raise ConnectionError("Write failed")
@@ -472,12 +492,12 @@ class ConnectionMinimal:
 
 class Connection(ConnectionMinimal):
     """
-    Complete server connection; in addition to 'ServerConnectionMinimal' provide
+    Complete server connection; in addition to 'ConnectionMinimal' provide
     a 'ServerInfo', which contains more information compared to the bare name/ip/addr
     of the server provided by 'ServerConnectionMinimal'.
-    The idea is tha 'ServerConnectionMinimal' is created for made the connection,
+    The idea is tha 'ConnectionMinimal' is created for made the connection,
     but than an info should be retrieved (via info, or if already got with a discover)
-    and the connection should be upgraded to a 'ServerConnection' for further uses.
+    and the connection should be upgraded to a 'Connection' for further uses.
     """
 
     def __init__(self,
@@ -496,4 +516,4 @@ class Connection(ConnectionMinimal):
         self.server_info["ip"] = server_ip
         self.server_info["port"] = server_port
 
-        log.d("Server connection info: \n%s", j(self.server_info))
+        log.d(f"Server connection info: \n{j(self.server_info)}")
